static int junle_num=146;

static char* junle_table[]=
{
   		  "Blues", 
             	  "Classic Rock",
                  "Country",
                  "Dance",
                  "Disco",
                  "Funk",
                  "Grunge",
                  "Hip-Hop", 
                  "Jazz",
                  "Metal",
                  "New Age",
                  "Oldies",
                  "Other", 
                  "Pop",
                  "R&B",
                  "Rap",
                  "Reggae", 
                  "Rock",
                  "Techno",
                  "Industrial",
                  "Alternative", 
                  "Ska",
                  "Death Metal", 
                  "Pranks",
                  "Soundtrack",
                  "Euro-Techno", 
                  "Ambient",
                  "Trip-Hop", 
                  "Vocal",
                  "Jazz+Funk", 
                  "Fusion",
                  "Trance",
                  "Classical",
                  "Instrumental", 
                  "Acid",
                  "House",
                  "Game",
                  "Sound Clip", 
                  "Gospel",
                  "Noise",
                  "Alt. Rock", 
                  "Bass",
                  "Soul",
                  "Punk",
                  "Space",
                  "Meditative",
                  "Instrumental Pop",
                  "Instrumental Rock", 
                  "Ethnic",
                  "Gothic",
                  "Darkwave",
                  "Techno-Industrial", 
                  "Electronic", 
                  "Pop-Folk",
                  "Eurodance", 
                  "Dream",
                  "Southern Rock", 
                  "Comedy", 
                  "Cult",
                  "Gangsta",
                  "Top 40",
                  "Christian Rap", 
                  "Pop/Funk", 
                  "Jungle",
                  "Native American", 
                  "Cabaret",
                  "New Wave",
                  "Psychadelic", 
                  "Rave",
                  "Showtunes", 
                  "Trailer",
                  "Lo-Fi",
                  "Tribal",
                  "Acid Punk",
                  "Acid Jazz", 
                  "Polka",
                  "Retro",
                  "Musical",
                  "Rock & Roll", 
                  "Hard Rock", 
                  "Folk",
                  "Folk/Rock",
                  "National Folk", 
                  "Swing",
                  "Fusion",
                  "Bebob",
                  "Latin",
                  "Revival",
                  "Celtic",
                  "Bluegrass",
                  "Avantgarde",
                  "Gothic Rock",
                  "Progressive Rock",
                  "Psychedelic Rock", 
                  "Symphonic Rock", 
                  "Slow Rock",
                  "Big Band", 
                  "Chorus",
                  "Easy Listening", 
                  "Acoustic", 
                  "Humour",
                  "Speech",
                  "Chanson", 
                  "Opera",
                  "Chamber Music", 
                  "Sonata",
                  "Symphony",
                  "Booty Bass", 
                  "Primus",
                  "Porn Groove", 
                  "Satire",
                  "Slow Jam", 
                  "Club",
                  "Tango",
                  "Samba",
                  "Folklore",
		  "Ballad", 
                  "Power Ballad",
                  "Rhytmic Soul", 
                  "Freestyle", 
		  "Duet",
		  "Punk Rock",
		  "Drum Solo",
		  "Acapella",
		  "Euro-House",
		  "Dance Hall",
		  "Goa",
		  "Drum & Bass",
		  "Club-House",
		  "Hardcore",
		  "Terror",
		  "Indie",
		  "BritPop",
		  "Negerpunk",
		  "Polsk Punk",
		  "Beat",
		  "Christian Gangsta Rap",
		  "Heavy Metal",
		  "Black Metal",
		  "Crossover",
		  "Contemporary Christian",
		  "Christian Rock",
		  "Merengue",
		  "Salsa",
		  "Trash Metal",
		  "Anime"
};
