
gchar *label_title_txt = "タイトル";
gchar *label_artist_txt = "アーティスト";
gchar *label_album_txt = "アルバム";
gchar *label_year_txt = "年";
gchar *label_junle_txt = "ジャンル";
gchar *label_comment_txt = "コメント";

gchar *window_main_title_txt = "gmp3te";

gchar *button_save_txt = "保存";
gchar *button_quit_txt = "閉じる";

gchar *status_load_txt = "正常に読み込まれました";
gchar *status_loadfail_txt = "読み込みに失敗しました";
gchar *status_save_txt = "正常に保存されました";
gchar *status_savefail_txt = "保存できませんでした";
gchar *status_notloaded_txt = "ファイルが読み込まれていません";
gchar *status_context_txt = "現在の状態";
gchar *status_loadenpty_txt = "ID3タグがありません";

gchar *status_entry_er = "入力したデータが長すぎます";

char *usage="\ngmp3te [file.mp3]\n";

gchar *No_Junle = "未入力";

